package com.jiuqi.util.chinese;

import java.io.Serializable;
import java.util.ArrayList;

public class ChineseInt implements Cloneable, Serializable {
   private static final char[] INVALID_CHARS = new char[]{' ', '　'};
   private String intValue;

   public ChineseInt() {
      this.intValue = "0";
   }

   public ChineseInt(long value) {
      this.intValue = String.valueOf(value);
   }

   public ChineseInt(String value) {
      if (isIntString(value)) {
         this.intValue = value;
      } else {
         try {
            this.intValue = valueOf(value);
         } catch (Exception var2) {
            this.intValue = "0";
         }
      }

   }

   public long longValue() {
      return Long.parseLong(this.intValue);
   }

   public String chineseValue(boolean showUnit, boolean useBigChar) {
      return parseToChinese(this.intValue, showUnit, useBigChar);
   }

   public static String valueOf(String value) throws NotValidChineseNumberException, NoValidCharException {
      boolean negtive = false;
      StringSplitor chars = new StringSplitor(value, INVALID_CHARS);
      char thisChar = chars.getNextValidChar();
      if (thisChar == '(' || thisChar == '（') {
         thisChar = chars.getNextValidChar();
         if (thisChar != '负') {
            throw new NotValidChineseNumberException("字符'('后面的字符无效，必须是'负'！");
         }

         thisChar = chars.getNextValidChar();
         if (thisChar != ')' && thisChar != '）') {
            throw new NotValidChineseNumberException("字符'负'后面的字符无效，必须是')'！");
         }

         negtive = true;
      }

      if (!negtive) {
         chars.rollBack();
      }

      boolean hasUnit = false;
      thisChar = chars.getNextValidChar();
      if (ChineseDigit.isValidChineseDigit(thisChar)) {
         thisChar = chars.getNextValidChar();
         if (ChineseDigitUnit.isValidChineseDigitUnit(String.valueOf(thisChar))) {
            hasUnit = true;
         }

         chars.rollBack();
         chars.rollBack();
         long longValue = 0L;
         if (hasUnit) {
            int tempValue = 0;

            while(chars.hasValidChar()) {
               thisChar = chars.getNextValidChar();
               if (ChineseDigit.isValidChineseDigit(thisChar)) {
                  try {
                     tempValue = ChineseDigit.valueOf(thisChar);
                  } catch (NotValidChineseDigitException var11) {
                     throw new NotValidChineseNumberException("存在非法字符\"" + thisChar + "\"！");
                  }

                  if (!chars.hasValidChar()) {
                     longValue += (long)tempValue;
                  }
               } else {
                  if (!ChineseDigitUnit.isValidChineseDigitUnit(String.valueOf(thisChar))) {
                     throw new NotValidChineseNumberException("存在非法的字符\"" + thisChar + "\"！");
                  }

                  if (ChineseDigitUnit.isLargeUnit(thisChar)) {
                     longValue += (long)tempValue;
                     tempValue = 0;

                     try {
                        longValue *= Long.parseLong(ChineseDigitUnit.valueOf(String.valueOf(thisChar)));
                     } catch (NotValidChineseDigitUnitException var10) {
                        throw new NotValidChineseNumberException("存在非法字符\"" + thisChar + "\"！");
                     }
                  } else {
                     try {
                        longValue += (long)tempValue * Long.parseLong(ChineseDigitUnit.valueOf(String.valueOf(thisChar)));
                     } catch (NotValidChineseDigitUnitException var9) {
                        throw new NotValidChineseNumberException("存在非法字符\"" + thisChar + "\"！");
                     }

                     tempValue = 0;
                  }
               }
            }
         } else {
            StringBuffer stringValue = new StringBuffer();

            while(chars.hasValidChar()) {
               thisChar = chars.getNextValidChar();
               if (ChineseDigit.isValidChineseDigit(thisChar)) {
                  try {
                     stringValue.append(ChineseDigit.valueOf(thisChar));
                  } catch (NotValidChineseDigitException var8) {
                     throw new NotValidChineseNumberException("存在非法字符\"" + thisChar + "\"！");
                  }
               }
            }

            longValue = Long.parseLong(stringValue.toString());
         }

         if (negtive) {
            longValue = -longValue;
         }

         return String.valueOf(longValue);
      } else {
         throw new NotValidChineseNumberException("开头的字符\"" + thisChar + "\"不是数字！");
      }
   }

   public static String parseToChinese(String value, boolean showUnit, boolean useBigChar) {
      if (value != null && !value.equals("") && isIntString(value)) {
         boolean negative = false;
         if (value.startsWith("-")) {
            negative = true;
            value = value.substring(1);
         }

         ArrayList digits = new ArrayList();

         int cursor;
         for(int i = 0; i < value.length(); ++i) {
            cursor = value.charAt(i) - 48;
            if (cursor >= 0 && cursor <= 9) {
               digits.add(new Integer(cursor));
            }
         }

         StringBuffer result = new StringBuffer();
         int smallInt;
         if (!showUnit) {
            for(cursor = 0; cursor < digits.size(); ++cursor) {
               smallInt = (Integer)digits.get(cursor);
               result.append(ChineseDigit.toChineseChar(smallInt, useBigChar));
            }
         } else {
            cursor = digits.size();
            int smallInt = false;

            for(int i = 0; cursor >= 0; ++i) {
               smallInt = getSmallInt(digits, i);
               if (smallInt != 0) {
                  result.insert(0, smallIntToChineseString(smallInt, useBigChar) + ChineseDigitUnit.toChineseUnit(getUnit(i * 4), useBigChar));
               }

               cursor -= 4;
               if (smallInt < 1000 && smallInt > 0 && cursor >= 0) {
                  result.insert(0, ChineseDigit.toChineseChar(0, useBigChar));
               }
            }
         }

         if (negative) {
            result.insert(0, "(负)");
         }

         return result.toString();
      } else {
         throw new IllegalArgumentException("字符串\"" + value + "\"不是合法的阿拉伯数字字符串！");
      }
   }

   private static int getSmallInt(ArrayList digits, int i) {
      int value = 0;
      int j = 0;

      for(int cursor = digits.size() - i * 4 - 1; j < 4 && cursor >= 0; ++j) {
         int digit = (Integer)digits.get(cursor);
         value = (int)((double)value + (double)digit * Math.pow(10.0, (double)j));
         --cursor;
      }

      return value;
   }

   private static String smallIntToChineseString(int value, boolean useBigChar) {
      if (value >= 0 && value < 10000) {
         int[] digits = new int[4];

         for(int count = 3; value != 0; --count) {
            digits[count] = value % 10;
            value /= 10;
         }

         StringBuffer result = new StringBuffer();
         boolean allZeroBefore = true;
         boolean zeroExist = false;

         for(int i = 0; i < digits.length; ++i) {
            if (digits[i] != 0) {
               result.append(ChineseDigit.toChineseChar(digits[i], useBigChar) + ChineseDigitUnit.toChineseUnit(getUnit(3 - i), useBigChar));
               allZeroBefore = false;
               zeroExist = false;
            } else if (!allZeroBefore) {
               boolean allZeroAfter = true;

               for(int j = i + 1; j < 4; ++j) {
                  if (digits[j] != 0) {
                     allZeroAfter = false;
                     break;
                  }
               }

               if (!allZeroAfter && !zeroExist) {
                  result.append(ChineseDigit.toChineseChar(0, useBigChar));
                  zeroExist = true;
               }
            }
         }

         return result.toString();
      } else {
         throw new IllegalArgumentException("只能解析0-9999的整数");
      }
   }

   static boolean isIntString(String value) {
      if (value.startsWith("-")) {
         value = value.substring(1);
      }

      return ChineseDigitUnit.isDigitString(value);
   }

   private static String getUnit(int i) {
      StringBuffer sb = new StringBuffer("1");

      for(int j = 0; j < i; ++j) {
         sb.append("0");
      }

      return sb.toString();
   }

   public Object clone() {
      return new ChineseInt(this.intValue);
   }

   public boolean equals(Object o) {
      if (o == this) {
         return true;
      } else if (!(o instanceof ChineseInt)) {
         return false;
      } else {
         ChineseInt value = (ChineseInt)o;
         return this.intValue.equals(value.intValue);
      }
   }

   public int hashCode() {
      int result = super.hashCode();
      if (this.intValue != null) {
         result ^= this.intValue.hashCode();
      }

      return result;
   }

   public String toString() {
      return this.chineseValue(true, false);
   }
}
